/*******************************************************************************
 Projektname:       LCD_NIKO_CC.cprj
 Bentigte Libss:  IntFunc_lib.cc
 Routinen:          lcd.cc, PRO-BOT128C_Lib_V2.cc
 Autor:             Niko
 Datum:             26. 11. 2012

 Funktion:          LC-Display ansteuern
*******************************************************************************/
// I2C-Adresse
#define I2C_ADDR_LCD 68

//Steuerzeichen
#define DISPLAY_MODE 0x28 //Modus: 4-Bit-Interface, 2 Zeilen, 5*7 Font
#define CURSOR_MODE 0x0F //Display an, Cursor an, Blinken
#define CURSOR_HOME 0x02
#define CURSOR_SCROLL_RIGHT 0x14
#define DISPLAY_CLEAR 0x01

// Nur whrend der Initialisierung im 8-Bit-Mode zu verwendende Befehle:
//Schaltet erstmal definiert den 8-Bit-Modus ein)
#define SOFT_RESET 0x03 // Soft-Reset bei Initialisierung
#define FOUR_BIT_MODE 0x02 // Eigentlich msste es 0x20 sein...

// Physische Pins des Pollin-LC-Displays:
#define RS_PIN 0x10
#define RW_PIN 0x20
#define ENABLE_PIN 0x40
#define BACKLIGHT_PIN 0x80

#define HIGH_ZERO_MASK 0x0F; // Zur Maskierung der oberen 4 Bit bei byte-daten

// Wartezeiten (Die HD44780-Chips des LC-Displays sind am Anfang etwas "lahm"
#define WAIT1 500
#define WAIT2 50
#define WAIT3 10

#define ON 1
#define OFF 0

byte backlight_status;

void main(void)
{

PRO_BOT128_INIT();    //PRO-BOT128 Setup

backlight_status=OFF;

// Display initialisieren:

int i;
	for (i=0; i<4; i++)
	{
	//3 * Soft-Reset - laut Datenblatt unbedingt vorgeschrieben!
	PCF8574_W(I2C_ADDR_LCD, ENABLE_PIN | SOFT_RESET); // Enable HIGH schalten und High-Nibbel "3" senden
    DELAY_MS(WAIT2);
	PCF8574_W(I2C_ADDR_LCD, SOFT_RESET); // Enable LOW schalten
	DELAY_MS(WAIT2);
	}

	//Der erste Schreibzugriff nach dem Reset ist ein 8 Bit-
	//Zugriff!! Also nur einmal schreiben, noch nicht Low-High Nibble!
	//4-Bit-Modus einstellen:
	PCF8574_W(I2C_ADDR_LCD, FOUR_BIT_MODE | ENABLE_PIN); // Enable HIGH schalten und High-Nibbel "2" senden
    DELAY_MS(WAIT2);
	PCF8574_W(I2C_ADDR_LCD, FOUR_BIT_MODE); // Enable LOW schalten
	DELAY_MS(WAIT2);

	// Ab hier HIGH- und LOW-Nibbel schreiben (4-Bit-Mode) - mit der Funktion sendcommand()
	// Funktionen des Displays festlegen:
	sendcommand(DISPLAY_MODE);

	DELAY_MS(WAIT2);

	sendcommand(CURSOR_MODE);

	DELAY_MS(WAIT2);

	BacklightOn();

    AbsDelay(1000);       //Wait 1Sec.

    sendcommand(DISPLAY_CLEAR);
    sendcommand(CURSOR_HOME);

    for(i=34; i<50; i++)
    {

            sendchar(i);
    }

        do                //Endless Loop
     {


     } while (1);

}

sendcommand(byte command)
{
	byte High_Nibble;
    byte Low_Nibble;
    byte Send_Data;

	High_Nibble = command >> 4; // 4 Bit Rechtsverschiebung
	High_Nibble = High_Nibble & HIGH_ZERO_MASK; //Obere 4 Bit mit 0 maskieren - nur wichtig bei Datentypen > 8 Bit!
	Low_Nibble = command &  HIGH_ZERO_MASK; //Obere 4 Bit mit 0 maskieren
	if (backlight_status == OFF)
		Send_Data = High_Nibble | ENABLE_PIN; // nur Enable setzen
	else
		Send_Data = High_Nibble | ENABLE_PIN | BACKLIGHT_PIN; // Enable + Backlight setzen
	PCF8574_W(I2C_ADDR_LCD, Send_Data);
	DELAY_MS(WAIT3);
	if (backlight_status == OFF)
		Send_Data = High_Nibble; //Enable auf LOW
	else
		Send_Data = High_Nibble | BACKLIGHT_PIN; //Enable auf LOW
	PCF8574_W(I2C_ADDR_LCD, Send_Data);//Enable Datenbernahme
	DELAY_MS(WAIT3);
	if (backlight_status == OFF)
		Send_Data = Low_Nibble | ENABLE_PIN; // Enable HIGH setzen
	else
		Send_Data = Low_Nibble | ENABLE_PIN | BACKLIGHT_PIN; // Enable HIGH setzen
	PCF8574_W(I2C_ADDR_LCD, Send_Data);
	DELAY_MS(WAIT3);
	if (backlight_status == OFF)
		Send_Data = Low_Nibble | RS_PIN; //Enable auf LOW
	else
		Send_Data = Low_Nibble | RS_PIN | BACKLIGHT_PIN; //Enable auf LOW und Backlight auf HIGH
	PCF8574_W(I2C_ADDR_LCD, Send_Data);//Enable Datenbernahme
	DELAY_MS(WAIT3);
    return 0;
}

BacklightOn(void)
{
backlight_status = ON;
sendcommand(DISPLAY_CLEAR | BACKLIGHT_PIN);
return 0;
}

BacklightOff(void)
{
backlight_status = OFF;
sendcommand(DISPLAY_CLEAR);
return 0;
}

sendchar(char character)
{
	byte High_Nibble;
	byte Low_Nibble;
	byte Send_Data;

	High_Nibble = character>> 4; // 4 Bit Rechtsverschiebung
	High_Nibble = High_Nibble & HIGH_ZERO_MASK; //Obere 4 Bit mit 0 maskieren - nur wichtig bei Datentypen > 8 Bit!
	Low_Nibble = character &  HIGH_ZERO_MASK; //Obere 4 Bit mit 0 maskieren
	if (backlight_status == OFF)
		Send_Data = High_Nibble | ENABLE_PIN | RS_PIN; // Enable und RS High setzen
	else
		Send_Data = High_Nibble | ENABLE_PIN | RS_PIN | BACKLIGHT_PIN; // Enable und RS High setzen
	PCF8574_W(I2C_ADDR_LCD, Send_Data);
	DELAY_MS(WAIT3);
	if (backlight_status == OFF)
		Send_Data = High_Nibble | RS_PIN; //Enable auf LOW; RS muss HIGH bleiben!
	else
		Send_Data = High_Nibble | RS_PIN | BACKLIGHT_PIN; //Enable auf LOW; RS muss HIGH bleiben!
	PCF8574_W(I2C_ADDR_LCD, Send_Data);//Enable Datenbernahme
	DELAY_MS(WAIT3);
	if (backlight_status == OFF)
		Send_Data = Low_Nibble | ENABLE_PIN | RS_PIN; // Enable und RS High setzen
	else
		Send_Data = Low_Nibble | ENABLE_PIN | RS_PIN | BACKLIGHT_PIN;  // Enable und RS High setzen
	PCF8574_W(I2C_ADDR_LCD, Send_Data);
	DELAY_MS(WAIT3);
	if (backlight_status == OFF)
		Send_Data = Low_Nibble | RS_PIN; //Enable auf LOW; RS muss HIGH bleiben!
	else
		Send_Data = Low_Nibble | RS_PIN | BACKLIGHT_PIN; //Enable auf LOW; RS muss HIGH bleiben!
	PCF8574_W(I2C_ADDR_LCD, Send_Data);//Enable Datenbernahme
	DELAY_MS(WAIT3);
    return 0;
}
